// src/components/providers/query-provider.tsx
'use client';

import {
  QueryClient,
  QueryClientProvider,
  QueryCache,
  MutationCache,
} from '@tanstack/react-query';
import { useState } from 'react';
import { toast } from 'sonner';

// Define error codes for different types of errors
export const enum QueryErrorCodes {
  USERS_FETCH_FAILED = 'USERS_FETCH_FAILED',
  STATUS_UPDATE_FAILED = 'STATUS_UPDATE_FAILED',
  // Add more as needed
}

const queryCacheOnError = (error: unknown, query: any) => {
  const errorMessage =
    error instanceof Error ? error.message : 'Something went wrong';

  switch (query.meta?.errCode) {
    case QueryErrorCodes.USERS_FETCH_FAILED:
      // Don't show toast for user fetch errors - handle in component
      break;
    default:
      // Only show generic errors for unexpected failures
      console.error('Query error:', error);
      break;
  }
};

const mutationCacheOnError = (
  error: unknown,
  variables: unknown,
  context: unknown,
  mutation: any,
) => {
  const errorMessage =
    error instanceof Error ? error.message : 'Something went wrong';

  switch (mutation.meta?.errCode) {
    case QueryErrorCodes.STATUS_UPDATE_FAILED:
      toast.error(`Failed to update status: ${errorMessage}`);
      break;
    default:
      toast.error(`Operation failed: ${errorMessage}`);
      break;
  }
};

type QueryProviderProps = {
  children: React.ReactNode;
};

export const QueryProvider = ({ children }: QueryProviderProps) => {
  const [queryClient] = useState(
    () =>
      new QueryClient({
        queryCache: new QueryCache({
          onError: queryCacheOnError,
        }),
        mutationCache: new MutationCache({
          onError: mutationCacheOnError,
        }),
        defaultOptions: {
          queries: {
            staleTime: 30 * 1000, // 30 seconds
            refetchOnWindowFocus: true,
            retry: (failureCount, error) => {
              // Don't retry on 4xx errors
              if (error instanceof Error && error.message.includes('4')) {
                return false;
              }
              return failureCount < 3;
            },
          },
          mutations: {
            retry: 1,
          },
        },
      }),
  );

  return (
    <QueryClientProvider client={queryClient}>{children}</QueryClientProvider>
  );
};
