'use client';
import { useState, useEffect, useRef } from 'react';
import { useAuth, useTVMode } from '@/components/context';
import {
  getRecentUsersWithStatuses,
  updateStatuses,
  updateUserStatus,
  type UserWithStatus,
} from '@/lib/hooks';
import { BasedAvatar, Drawer, DrawerTrigger, Loading } from '@/components/ui';
import { StatusMessage, SubmitButton } from '@/components/default';
import { toast } from 'sonner';
import { HistoryDrawer } from '@/components/status';
import type { Profile } from '@/utils/supabase';
import { makeConditionalClassName } from '@/lib/utils';
import { Card, CardContent, CardHeader } from '@/components/ui/card';
import { Badge } from '@/components/ui/badge';
import { Checkbox } from '@/components/ui/checkbox';
import { Input } from '@/components/ui/input';
import { Button } from '@/components/ui/button';
import { RefreshCw, Clock, Wifi, WifiOff, Calendar } from 'lucide-react';
import { useQuery, useMutation, useQueryClient } from '@tanstack/react-query';
import { QueryErrorCodes } from '@/components/context';
import { createClient } from '@/utils/supabase';
import type { RealtimeChannel } from '@supabase/supabase-js';

type ListProps = { initialStatuses: UserWithStatus[] };

export const StatusList = ({ initialStatuses = [] }: ListProps) => {
  const { isAuthenticated } = useAuth();
  const { tvMode } = useTVMode();
  const queryClient = useQueryClient();

  const [selectedUsers, setSelectedUsers] = useState<UserWithStatus[]>([]);
  const [selectAll, setSelectAll] = useState(false);
  const [statusInput, setStatusInput] = useState('');
  const [selectedHistoryUser, setSelectedHistoryUser] =
    useState<Profile | null>(null);
  const [updateStatusMessage, setUpdateStatusMessage] = useState('');

  const [connectionStatus, setConnectionStatus] = useState<
    'connecting' | 'connected' | 'disconnected' | 'updating'
  >('connecting');
  const [newStatuses, setNewStatuses] = useState<Set<UserWithStatus>>(new Set());
  const channelRef = useRef<RealtimeChannel | null>(null);
  const supabaseRef = useRef(createClient());

  const {
    data: usersWithStatuses = initialStatuses,
    isLoading: loading,
    error,
    refetch,
    isFetching,
    dataUpdatedAt,
  } = useQuery({
    queryKey: ['users-with-statuses'],
    queryFn: async () => {
      try {
        const response = await getRecentUsersWithStatuses();
        if (!response.success) throw new Error(response.error);
        return response.data;
      } catch (error) {
        toast.error(`Error fetching technicians: ${error as Error}`)
        throw error;
      }
    },
    enabled: isAuthenticated,
    refetchInterval: 30000, // 30 Second interval as we should rely on subscription.
    refetchOnWindowFocus: true,
    refetchOnMount: true,
    initialData: initialStatuses,
    meta: { errCode: QueryErrorCodes.USERS_FETCH_FAILED },
  });

  useEffect(() => {
    if (!isAuthenticated) return;

    const maxReconnectAttempts = 3;
    let reconnectAttempts = 0;
    let reconnectTimeout: NodeJS.Timeout;
    let isComponentMounted = true;
    let currentChannel: RealtimeChannel | null = null;

    const setUpRealtimeConnection = () => {
      if (!isComponentMounted) return;

      if (currentChannel) {
        supabaseRef.current.removeChannel(currentChannel)
          .catch((error) => {
            setConnectionStatus('disconnected');
            console.error(`Error unsubscribing: ${error}`);
          })
        currentChannel = null;
      }

      setConnectionStatus('connecting');
      const channel = supabaseRef.current
        .channel('status_updates')
        .on('broadcast', { event: 'status_updated' }, () => {
          refetch().catch((error) => {
            console.error(`Error refetching statuses: ${error as Error}`)
          });
        })
        .subscribe((status) => {
          // eslint-disable-next-line @typescript-eslint/no-unsafe-enum-comparison
          if (status === 'SUBSCRIBED') {
            setConnectionStatus('connected');
            reconnectAttempts = 0;
          // eslint-disable-next-line @typescript-eslint/no-unsafe-enum-comparison
          } else if (status === 'CHANNEL_ERROR') {
            setConnectionStatus('disconnected');
          // eslint-disable-next-line @typescript-eslint/no-unsafe-enum-comparison
          } else if (status === 'CLOSED') {
            setConnectionStatus('disconnected')
            if (isComponentMounted && reconnectAttempts < maxReconnectAttempts) {
              reconnectAttempts++;
              const delay = 2000 * reconnectAttempts;
              console.log(
                `Reconnecting after close.
                  ${reconnectAttempts} attempts of
                  ${maxReconnectAttempts} in ${delay}ms
              `);
              if (reconnectTimeout) clearTimeout(reconnectTimeout);
              reconnectTimeout = setTimeout(() => {
                if (isComponentMounted) setUpRealtimeConnection();
              }, delay);
            } else {
              console.warn('Max reconnection attempts reached or component is not mounted.');
              setConnectionStatus('disconnected');
            }
          }
        });
      currentChannel = channel;
      channelRef.current = channel;
    };

    const initialTimeout = setTimeout(() => {
      if (isComponentMounted) setUpRealtimeConnection();
    }, 1000);

    return () => {
      isComponentMounted = false;
      if (initialTimeout) clearTimeout(initialTimeout);
      if (reconnectTimeout) clearTimeout(reconnectTimeout);
      if (currentChannel) {
        supabaseRef.current.removeChannel(currentChannel)
          .catch((error) => {
            console.error(`Error unsubscribing: ${error as Error}`)
          });
        channelRef.current = null;
      }
      setConnectionStatus('disconnected');
    }
  }, [isAuthenticated, refetch]);

  const updateStatusMutation = useMutation({

    mutationFn: async ({
      usersWithStatuses,
      status,
    }: {
      usersWithStatuses: UserWithStatus[];
      status: string;
    }) => {
      const previousConnectionStatus = connectionStatus;
      try {
        if (usersWithStatuses.length <= 0) {
          const result = await updateUserStatus(status);
          if (!result.success) throw new Error(result.error);
          return result.data;
        } else {
          const result = await updateStatuses(usersWithStatuses, status);
          if (!result.success) throw new Error(result.error);
          return result.data;
        }
      } catch (error) {
        console.error(`Error updating statuses: ${error as Error}`);
      }
    },
    meta: { errCode: QueryErrorCodes.STATUS_UPDATE_FAILED },

    onMutate: async ({ usersWithStatuses, status }) => {
      // Optimistic update logic
      await queryClient.cancelQueries({ queryKey: ['users-with-statuses']} );
      const previousData = queryClient.getQueryData<UserWithStatus[]>([
        'users-with-statuses',
      ]);
      if (previousData && usersWithStatuses.length > 0) {
        const now = new Date().toISOString();
        const optimisticData = previousData.map((userStatus) => {
          if (usersWithStatuses.some((selected) => selected.user.id === userStatus.user.id))
            return { ...userStatus, status, created_at: now};
          return userStatus;
        });
        queryClient.setQueryData(['users-with-statuses'], optimisticData);

        // Add animation to optimisticly updated statuses
        const updatedStatuses = usersWithStatuses;
        setNewStatuses((prev) => new Set([...prev, ...updatedStatuses]))
        setTimeout(() => {
          setNewStatuses((prev) => {
            const updated = new Set(prev);
            updatedStatuses.forEach((updatedStatus) => updated.delete(updatedStatus));
            return updated;
          });
        }, 1000)
      }
      return { previousData };
    },

    onSuccess: (data) => {
      queryClient.invalidateQueries({ queryKey: ['users-with-statuses']} )
        .catch((error) => console.error(`Error invalidating query: ${error}`));
      if (!data) return;
      data.forEach((statusUpdate) => {
        toast.success(`
          ${statusUpdate.user.full_name}'s status updated to '${statusUpdate.status}'.
        `);
      });
      setSelectedUsers([]);
      setStatusInput('');
    },

    onError: (error, _variables, context) => {
      if (context?.previousData)
        queryClient.setQueryData(['users-with-statuses'], context.previousData);
      toast.error(`Error updating statuses: ${error}`);
    },
  });

  const handleUpdateStatus = () => {
    if (!isAuthenticated) {
      setUpdateStatusMessage('You must be signed in to update technican statuses!')
      return;
    } else if (!statusInput.trim()) {
      setUpdateStatusMessage('Your status must be in between 3 & 80 characters long!');
      return;
    }
    updateStatusMutation.mutate({
      usersWithStatuses: selectedUsers,
      status: statusInput.trim(),
    });
  };


  const handleCheckboxChange = (user: UserWithStatus) => {
    setSelectedUsers((prev) =>
      prev.some((u) => u.user.id === user.user.id)
        ? prev.filter((prevUser) => prevUser.user.id !== user.user.id)
        : [...prev, user],
    );
  };

  const handleSelectAllChange = () => {
    if (selectAll) {
      setSelectedUsers([]);
    } else {
      setSelectedUsers(usersWithStatuses);
    }
    setSelectAll(!selectAll);
  };

  useEffect(() => {
    setSelectAll(
      selectedUsers.length === usersWithStatuses.length &&
        usersWithStatuses.length > 0,
    );
  }, [selectedUsers.length, usersWithStatuses.length]);

  const getConnectionIcon = () => {
    switch (connectionStatus) {
      case 'connected':
        return <Wifi className='w-4 h-4 text-green-500' />;
      case 'connecting':
        return <Wifi className='w-4 h-4 text-yellow-500 animate-pulse' />;
      case 'disconnected':
        return <WifiOff className='w-4 h-4 text-red-500' />;
      case 'updating':
        return <RefreshCw className='w-4 h-4 text-blue-500 animate-spin' />;
    }
  };

  const getConnectionText = () => {
    switch (connectionStatus) {
      case 'connected':
        return 'Connected';
      case 'connecting':
        return 'Connecting...';
      case 'disconnected':
        return 'Disconnected';
      case 'updating':
        return 'Updating...';
    }
  };

  const formatTime = (timestamp: string) => {
    const date = new Date(timestamp);
    const time = date.toLocaleTimeString('en-US', {
      hour: 'numeric',
      minute: 'numeric',
    });
    return `${time}`;
  };

  const formatDate = (timestamp: string) => {
    const date = new Date(timestamp);
    const day = date.getDate();
    const month = date.toLocaleString('default', { month: 'long' });
    return `${month} ${day}`;
  };

  if (loading) {
    return (
      <div className='flex justify-center items-center min-h-[400px]'>
        <Loading className='w-full' alpha={0.5} />
      </div>
    );
  }

  if (error) {
    return (
      <div className='flex flex-col justify-center items-center min-h-[400px] gap-4'>
        <p className='text-red-500'>Error loading status updates</p>
        <Button onClick={() => refetch()} variant='outline'>
          <RefreshCw className='w-4 h-4 mr-2' />
          Retry
        </Button>
      </div>
    );
  }

  const containerClassName = makeConditionalClassName({
    context: tvMode,
    defaultClassName: 'flex flex-col mx-auto space-y-4 items-center',
    on: 'lg:w-11/12 w-full mt-15',
    off: 'sm:w-5/6 md:3/4 lg:w-1/2',
  });

  const headerClassName = makeConditionalClassName({
    context: tvMode,
    defaultClassName: 'w-full',
    on: 'hidden',
    off: 'flex mb-4 justify-between',
  });

  const cardContainerClassName = makeConditionalClassName({
    context: tvMode,
    defaultClassName: '',
    on: '',
    off: 'space-y-3 items-center justify-center w-full',
  });

  const cardClassName = makeConditionalClassName({
    context: tvMode,
    defaultClassName:
      'transition-all duration-300 hover:shadow-md hover:bg-muted/50 cursor-pointer',
    on: 'lg:text-4xl',
    off: 'lg:text-base lg:w-full',
  });

  return (
    <div className={containerClassName}>

      <div className={headerClassName}>
        <div className='flex items-center gap-2'>
          <Checkbox
            id='select-all'
            checked={selectAll}
            onCheckedChange={handleSelectAllChange}
          />
          <label htmlFor='select-all' className='text-sm font-medium'>
            Select All
          </label>
        </div>
        <div className='flex items-center gap-2'>
          <Badge variant='outline' className='flex items-center gap-2'>
            {getConnectionIcon()}
            <span className='text-sm'>{getConnectionText()}</span>
          </Badge>
        </div>
      </div>

      <div className={cardContainerClassName}>
        {usersWithStatuses.map((userWithStatus) => {
          const isSelected = selectedUsers.some((u) => u.user.id === userWithStatus.user.id);
          const isNewStatus = newStatuses.has(userWithStatus);
          const isUpdatedByOther = userWithStatus.updated_by &&
            userWithStatus.updated_by.id !== userWithStatus.user.id;
          return (
            <Card
              key={userWithStatus.user.id}
              className={`
                ${cardClassName}
                ${isSelected ? 'ring2 ring-primary' : ''}
                ${isNewStatus ? 'animate-pulse bg-primary/5 border-primary/20' : ''}
              `}
            >
              <CardHeader className='pb-3'>
                <div className='flex items-center justify-between'>
                  <div className='flex items-center gap-3'>
                    {!tvMode && (
                      <Checkbox
                        checked={isSelected}
                        onCheckedChange={() => handleCheckboxChange(userWithStatus)}
                        onClick={(e) => e.stopPropagation()}
                      />
                    )}
                    <BasedAvatar
                      src={userWithStatus.user.avatar_url}
                      fullName={userWithStatus.user.full_name}
                      className={tvMode ? 'w-24 h-24' : 'w-16 h-16'}
                    />
                    <div className='my-auto'>
                      <h3 className={`font-semibold ${tvMode ? 'text-5xl' : 'text-2xl'}`}>
                        {userWithStatus.user.full_name}
                      </h3>
                      {isUpdatedByOther && (
                        <div className='flex items-center gap-1 text-muted-foreground'>
                          <BasedAvatar
                            src={userWithStatus.updated_by?.avatar_url}
                            fullName={userWithStatus.updated_by?.full_name}
                            className='w-5 h-5'
                          />
                          <span className={`${tvMode ? 'text-3xl' : 'text-sm'}`}>
                            Updated by {userWithStatus.updated_by?.full_name}
                          </span>
                        </div>
                      )}
                    </div>
                  </div>
                  <div className='my-auto'>
                    <div className='flex items-center gap-2 text-muted-foreground'>
                      <Clock className={`${tvMode ? 'w-8 h-8' : 'w-6 h-6'}`} />
                      <span className={`${tvMode ? 'text-3xl' : 'text-xl'}`}>
                        {formatTime(userWithStatus.created_at)}
                      </span>
                    </div>
                    <div className='flex items-center gap-2 text-muted-foreground'>
                      <Calendar className={`${tvMode ? 'w-8 h-8' : 'w-6 h-6'}`} />
                      <span className={`${tvMode ? 'text-3xl' : 'text-xl'}`}>
                        {formatDate(userWithStatus.created_at)}
                      </span>
                    </div>
                  </div>
                </div>
              </CardHeader>

              <CardContent className='pt-0'>
                <Drawer>
                  <DrawerTrigger asChild>
                    <div
                      className={`
                        p-4 rounded-lg bg-muted/30 hover:bg-muted/50
                        transition-colors cursor-pointer text-left
                        ${tvMode ? 'text-4xl' : 'text-xl'}
                      `}
                      onClick={() => setSelectedHistoryUser(userWithStatus.user)}
                    >
                      <p className='font-medium'>{userWithStatus.status}</p>
                    </div>
                  </DrawerTrigger>
                    {selectedHistoryUser === userWithStatus.user && (
                      <HistoryDrawer user={selectedHistoryUser} />
                    )}
                </Drawer>
              </CardContent>
            </Card>
          );
        })}
      </div>

      {usersWithStatuses.length === 0 && (
        <Card className='p-8 text-center'>
          <p className={`text-muted-foreground ${tvMode ? 'text-4xl' : 'text-lg'}`}>
            No status updates have been made in the past day.
          </p>
        </Card>
      )}

    </div>
  );

};
