'use client';
import { useState, useEffect, useRef } from 'react';
import { useAuth, useTVMode } from '@/components/context';
import {
  getRecentUsersWithStatuses,
  updateStatuses,
  updateUserStatus,
  type UserWithStatus,
} from '@/lib/hooks';
import { BasedAvatar, Drawer, DrawerTrigger, Loading } from '@/components/ui';
import { SubmitButton } from '@/components/default';
import { toast } from 'sonner';
import { HistoryDrawer } from '@/components/status';
import type { Profile } from '@/utils/supabase';
import { makeConditionalClassName } from '@/lib/utils';
import { Card, CardContent, CardHeader } from '@/components/ui/card';
import { Badge } from '@/components/ui/badge';
import { Checkbox } from '@/components/ui/checkbox';
import { Input } from '@/components/ui/input';
import { Button } from '@/components/ui/button';
import { RefreshCw, Clock, Wifi, WifiOff } from 'lucide-react';
import { useQuery, useMutation, useQueryClient } from '@tanstack/react-query';
import { QueryErrorCodes } from '@/components/context';
import { createClient } from '@/utils/supabase';
import type { RealtimeChannel } from '@supabase/supabase-js';

type StatusListProps = {
  initialStatuses: UserWithStatus[];
};

export const StatusList = ({ initialStatuses = [] }: StatusListProps) => {
  // Fixed props destructuring
  const { isAuthenticated } = useAuth();
  const { tvMode } = useTVMode();
  const queryClient = useQueryClient();

  const [selectedUsers, setSelectedUsers] = useState<UserWithStatus[]>([]);
  const [selectAll, setSelectAll] = useState(false);
  const [statusInput, setStatusInput] = useState('');
  const [selectedHistoryUser, setSelectedHistoryUser] =
    useState<Profile | null>(null);

  const [connectionStatus, setConnectionStatus] = useState<
    'connecting' | 'connected' | 'disconnected'
  >('connecting');
  const [newStatusIds, setNewStatusIds] = useState<Set<string>>(new Set());
  const channelRef = useRef<RealtimeChannel | null>(null);
  const supabaseRef = useRef(createClient());

  // Keep all your existing React Query code exactly as is
  const {
    data: usersWithStatuses = initialStatuses,
    isLoading: loading,
    error,
    refetch,
    isFetching,
    dataUpdatedAt,
  } = useQuery({
    queryKey: ['users-with-statuses'],
    queryFn: async () => {
      try {
        const response = await getRecentUsersWithStatuses();
        if (!response.success) throw new Error(response.error);
        return response.data;
      } catch (error) {
        toast.error(
          `Error fetching technicians: ${error instanceof Error ? error.message : 'Unknown error'}`,
        );
        throw error;
      }
    },
    enabled: isAuthenticated,
    refetchInterval: 30000, // Changed to 30 seconds as backup
    refetchOnWindowFocus: true,
    refetchOnMount: true,
    initialData: initialStatuses,
    meta: { errCode: QueryErrorCodes.USERS_FETCH_FAILED },
  });

  // Add this new useEffect for realtime enhancement
  useEffect(() => {
    if (!isAuthenticated) return;

    let reconnectAttempts = 0;
    const maxReconnectAttempts = 3;
    let reconnectTimeout: NodeJS.Timeout;
    let isComponentMounted = true;
    let currentChannel: RealtimeChannel | null = null;

    const setUpRealtimeConnection = () => {
      if (!isComponentMounted) return;
      if (currentChannel) {
        supabaseRef.current.removeChannel(currentChannel).catch((error) => {
          console.error(`Error unsubscribing: ${error}`);
        });
        currentChannel = null;
      }
      setConnectionStatus('connecting');
      const channel = supabaseRef.current
        .channel('status_updates')
        .on('broadcast', { event: 'status_updated' }, (payload) => {
          console.log('Realtime update received, triggering refetch...');
          refetch().catch((error) => {
            console.error(`Error refetching: ${error}`);
          });
        })
        .subscribe((status) => {
          // eslint-disable-next-line @typescript-eslint/no-unsafe-enum-comparison
          if (status === 'SUBSCRIBED') {
            console.log('Realtime connection established');
            setConnectionStatus('connected');
            reconnectAttempts = 0;
            // eslint-disable-next-line @typescript-eslint/no-unsafe-enum-comparison
          } else if (status === 'CHANNEL_ERROR') {
            console.log('Realtime connection failed, relying on polling');
            setConnectionStatus('disconnected');
            // eslint-disable-next-line @typescript-eslint/no-unsafe-enum-comparison
          } else if (status === 'CLOSED') {
            console.log('Realtime connection closed');
            setConnectionStatus('disconnected');
            if (
              isComponentMounted &&
              reconnectAttempts < maxReconnectAttempts
            ) {
              reconnectAttempts++;
              const delay = 2000 * reconnectAttempts;

              console.log(
                `Reconnecting after close ${reconnectAttempts}/${maxReconnectAttempts} in ${delay}ms`,
              );

              if (reconnectTimeout) {
                clearTimeout(reconnectTimeout);
              }

              reconnectTimeout = setTimeout(() => {
                if (isComponentMounted) {
                  setUpRealtimeConnection();
                }
              }, delay);
            } else {
              console.log(
                'Max reconnection attempts reached or component unmounted',
              );
            }
          }
        });
      currentChannel = channel;
      channelRef.current = channel;
    };

    const initialTimeout = setTimeout(() => {
      if (isComponentMounted) {
        setUpRealtimeConnection();
      }
    }, 1000);

    return () => {
      isComponentMounted = false;
      if (initialTimeout) clearTimeout(initialTimeout);
      if (reconnectTimeout) clearTimeout(reconnectTimeout);

      if (currentChannel) {
        console.log('Cleaning up realtime connection...');
        supabaseRef.current.removeChannel(currentChannel).catch((error) => {
          console.error(`Error unsubscribing: ${error}`);
        });
        channelRef.current = null;
      }
    };
  }, [isAuthenticated, refetch]);

  // Updated mutation
  const updateStatusMutation = useMutation({
    mutationFn: async ({
      usersWithStatuses,
      status,
    }: {
      usersWithStatuses: UserWithStatus[];
      status: string;
    }) => {
      if (usersWithStatuses.length === 0) {
        const result = await updateUserStatus(status);
        if (!result.success) throw new Error(result.error);
        return { type: 'single', result };
      } else {
        const result = await updateStatuses(usersWithStatuses, status);
        if (!result.success) throw new Error(result.error);
        return { type: 'multiple', result, count: usersWithStatuses.length };
      }
    },
    meta: { errCode: QueryErrorCodes.STATUS_UPDATE_FAILED },
    onMutate: async ({ usersWithStatuses, status }) => {
      // Optimistic update logic
      await queryClient.cancelQueries({ queryKey: ['users-with-statuses'] });
      const previousData = queryClient.getQueryData<UserWithStatus[]>([
        'users-with-statuses',
      ]);

      if (previousData && usersWithStatuses.length > 0) {
        const now = new Date().toISOString();
        const optimisticData = previousData.map((userStatus) => {
          if (
            usersWithStatuses.some(
              (selected) => selected.user.id === userStatus.user.id,
            )
          ) {
            return {
              ...userStatus,
              status,
              created_at: now,
            };
          }
          return userStatus;
        });
        queryClient.setQueryData(['users-with-statuses'], optimisticData);

        // Add animation for optimistic updates
        const updatedIds = usersWithStatuses.map((u) => u.user.id);
        setNewStatusIds((prev) => new Set([...prev, ...updatedIds]));

        // Remove animation after 1 second
        setTimeout(() => {
          setNewStatusIds((prev) => {
            const updated = new Set(prev);
            updatedIds.forEach((id) => updated.delete(id));
            return updated;
          });
        }, 1000);
      }

      return { previousData };
    },
    onSuccess: (data) => {
      // Handle success in the mutation function
      void queryClient.invalidateQueries({ queryKey: ['users-with-statuses'] }); // Fixed floating promise

      if (data.type === 'single') {
        toast.success('Status updated for signed in user.');
      } else {
        toast.success(`Status updated for ${data.count} selected users.`);
      }

      setSelectedUsers([]);
      setStatusInput('');
    },
    onError: (error, _variables, context) => {
      // Fixed unused variables
      // Rollback optimistic update
      if (context?.previousData) {
        queryClient.setQueryData(['users-with-statuses'], context.previousData);
      }
      // Error handling is done in the global mutation cache
      console.error('Status update failed:', error);
    },
  });

  const handleUpdateStatus = () => {
    if (!isAuthenticated) {
      toast.error('You must be signed in to update statuses.');
      return;
    }
    if (!statusInput.trim()) {
      toast.error('Please enter a valid status.');
      return;
    }

    updateStatusMutation.mutate({
      usersWithStatuses: selectedUsers,
      status: statusInput.trim(),
    });
  };

  const handleCheckboxChange = (user: UserWithStatus) => {
    setSelectedUsers((prev) =>
      prev.some((u) => u.user.id === user.user.id)
        ? prev.filter((prevUser) => prevUser.user.id !== user.user.id)
        : [...prev, user],
    );
  };

  const handleSelectAllChange = () => {
    if (selectAll) {
      setSelectedUsers([]);
    } else {
      setSelectedUsers(usersWithStatuses);
    }
    setSelectAll(!selectAll);
  };

  useEffect(() => {
    setSelectAll(
      selectedUsers.length === usersWithStatuses.length &&
        usersWithStatuses.length > 0,
    );
  }, [selectedUsers.length, usersWithStatuses.length]);

  const getConnectionIcon = () => {
    switch (connectionStatus) {
      case 'connected':
        return <Wifi className='w-4 h-4 text-green-500' />;
      case 'connecting':
        return <Wifi className='w-4 h-4 text-yellow-500 animate-pulse' />;
      case 'disconnected':
        return <WifiOff className='w-4 h-4 text-red-500' />;
    }
  };

  const getConnectionText = () => {
    switch (connectionStatus) {
      case 'connected':
        return 'Connected';
      case 'connecting':
        return 'Connecting...';
      case 'disconnected':
        return 'Disconnected';
    }
  };

  const formatTime = (timestamp: string) => {
    const date = new Date(timestamp);
    const time = date.toLocaleTimeString('en-US', {
      hour: 'numeric',
      minute: 'numeric',
    });
    const day = date.getDate();
    const month = date.toLocaleString('default', { month: 'long' });
    return `${time} - ${month} ${day}`;
  };

  if (loading) {
    return (
      <div className='flex justify-center items-center min-h-[400px]'>
        <Loading className='w-full' alpha={0.5} />
      </div>
    );
  }

  if (error) {
    return (
      <div className='flex flex-col justify-center items-center min-h-[400px] gap-4'>
        <p className='text-red-500'>Error loading status updates</p>
        <Button onClick={() => refetch()} variant='outline'>
          <RefreshCw className='w-4 h-4 mr-2' />
          Retry
        </Button>
      </div>
    );
  }

  const containerClassName = makeConditionalClassName({
    context: tvMode,
    defaultClassName: 'mx-auto space-y-4',
    on: 'lg:w-11/12 w-full mt-15',
    off: 'w-5/6',
  });

  const cardClassName = makeConditionalClassName({
    context: tvMode,
    defaultClassName: 'transition-all duration-300 hover:shadow-md',
    on: 'lg:text-4xl',
    off: 'lg:text-base',
  });

  return (
    <div className={containerClassName}>
      {/* Status Header */}
      <div className='flex items-center justify-between mb-6'>
        <div className='flex items-center gap-2'>
          <h2 className={`font-bold ${tvMode ? 'text-6xl' : 'text-2xl'}`}>
            Tech Status
          </h2>
          {isFetching ? (
            <Badge variant='outline' className='flex items-center gap-2'>
              <RefreshCw className='w-4 h-4 animate-spin text-blue-500' />
              <span className='text-xs'>Updating...</span>
            </Badge>
          ) : (
            <Badge variant='outline' className='flex items-center gap-2'>
              {getConnectionIcon()}
              <span className='text-xs'>{getConnectionText()}</span>
            </Badge>
          )}
        </div>
        {!tvMode && usersWithStatuses.length > 0 && (
          <div className='flex items-center gap-2'>
            <Checkbox
              id='select-all'
              checked={selectAll}
              onCheckedChange={handleSelectAllChange}
            />
            <label htmlFor='select-all' className='text-sm font-medium'>
              Select All ({selectedUsers.length} selected)
            </label>
          </div>
        )}
      </div>

      {/* Status Cards */}
      <div className='space-y-3'>
        {usersWithStatuses.map((userWithStatus) => {
          const isSelected = selectedUsers.some(
            (u) => u.user.id === userWithStatus.user.id,
          );
          const isNewStatus = newStatusIds.has(userWithStatus.user.id);
          const isUpdatedByOther =
            userWithStatus.updated_by &&
            userWithStatus.updated_by.id !== userWithStatus.user.id;

          return (
            <Card
              key={userWithStatus.user.id}
              className={`
                ${cardClassName}
                ${isSelected ? 'ring-2 ring-primary' : ''}
                ${isNewStatus ? 'animate-pulse bg-primary/5 border-primary/20' : ''}
                hover:bg-muted/50 cursor-pointer
              `}
            >
              <CardHeader className='pb-3'>
                <div className='flex items-center justify-between'>
                  <div className='flex items-center gap-3'>
                    {!tvMode && (
                      <Checkbox
                        checked={isSelected}
                        onCheckedChange={() =>
                          handleCheckboxChange(userWithStatus)
                        }
                        onClick={(e) => e.stopPropagation()}
                      />
                    )}
                    <BasedAvatar
                      src={userWithStatus.user.avatar_url}
                      fullName={userWithStatus.user.full_name}
                      className={tvMode ? 'w-16 h-16' : 'w-12 h-12'}
                    />
                    <div>
                      <h3
                        className={`font-semibold ${tvMode ? 'text-5xl' : 'text-lg'}`}
                      >
                        {userWithStatus.user.full_name ?? 'Unknown User'}
                      </h3>
                      {isUpdatedByOther && (
                        <div className='flex items-center gap-1 text-muted-foreground'>
                          <BasedAvatar
                            src={userWithStatus.updated_by?.avatar_url}
                            fullName={userWithStatus.updated_by?.full_name}
                            className='w-3 h-3'
                          />
                          {userWithStatus.updated_by && (
                            <span
                              className={`text-xs ${tvMode ? 'text-3xl' : ''}`}
                            >
                              Updated by {userWithStatus.updated_by.full_name}
                            </span>
                          )}
                        </div>
                      )}
                    </div>
                  </div>
                  <div className='flex items-center gap-2 text-muted-foreground'>
                    <Clock className={`${tvMode ? 'w-8 h-8' : 'w-4 h-4'}`} />
                    <span className={`text-sm ${tvMode ? 'text-3xl' : ''}`}>
                      {formatTime(userWithStatus.created_at)}
                    </span>
                  </div>
                </div>
              </CardHeader>
              <CardContent className='pt-0'>
                <Drawer>
                  <DrawerTrigger asChild>
                    <div
                      className={`
                        p-4 rounded-lg bg-muted/30 hover:bg-muted/50
                        transition-colors cursor-pointer text-left
                        ${tvMode ? 'text-4xl' : 'text-base'}
                      `}
                      onClick={() =>
                        setSelectedHistoryUser(userWithStatus.user)
                      }
                    >
                      <p className='font-medium'>{userWithStatus.status}</p>
                    </div>
                  </DrawerTrigger>
                  {selectedHistoryUser === userWithStatus.user && (
                    <HistoryDrawer user={selectedHistoryUser} />
                  )}
                </Drawer>
              </CardContent>
            </Card>
          );
        })}
      </div>

      {usersWithStatuses.length === 0 && (
        <Card className='p-8 text-center'>
          <p
            className={`text-muted-foreground ${tvMode ? 'text-4xl' : 'text-lg'}`}
          >
            No status updates yet
          </p>
        </Card>
      )}

      {/* Status Update Input */}
      {!tvMode && (
        <Card className='p-6 mt-6'>
          <div className='flex flex-col gap-4'>
            <h3 className='text-lg font-semibold'>Update Status</h3>
            <div className='flex gap-4'>
              <Input
                autoFocus
                type='text'
                placeholder="What's your status?"
                className='flex-1 text-base'
                value={statusInput}
                onChange={(e) => setStatusInput(e.target.value)}
                onKeyDown={(e) => {
                  if (
                    e.key === 'Enter' &&
                    !e.shiftKey &&
                    !updateStatusMutation.isPending
                  ) {
                    e.preventDefault();
                    handleUpdateStatus();
                  }
                }}
                disabled={updateStatusMutation.isPending}
              />
              <SubmitButton
                onClick={handleUpdateStatus}
                disabled={updateStatusMutation.isPending}
                className='px-6'
              >
                {updateStatusMutation.isPending
                  ? 'Updating...'
                  : selectedUsers.length > 0
                    ? `Update ${selectedUsers.length} Users`
                    : 'Update Status'}
              </SubmitButton>
            </div>
            {selectedUsers.length > 0 && (
              <p className='text-sm text-muted-foreground'>
                Updating status for {selectedUsers.length} selected users
              </p>
            )}
          </div>
        </Card>
      )}

      {/* Global Status History Drawer */}
      <div className='flex justify-center mt-6'>
        <Drawer>
          <DrawerTrigger asChild>
            <Button variant='outline' className={tvMode ? 'text-3xl p-6' : ''}>
              View All Status History
            </Button>
          </DrawerTrigger>
          <HistoryDrawer />
        </Drawer>
      </div>
    </div>
  );
};
